/* Copyright (c) Sun Microsystems 2006. All rights reserved.
   This file is part of OpenOffice.org project.
   
   Written by Ramón García Fernández <ramon.garcia.f+openoffice@gmail.com>.
   This work was supported by a Google Summer of Code grant.
*/

/* Code for writting object information in Basic syntax */

#include "browser_lang.hxx"
#include <rtl/ustrbuf.hxx>
#include <com/sun/star/beans/PropertyAttribute.hpp>

namespace objectbrowser_impl {
using namespace ::rtl;
using namespace ::com::sun::star::reflection;
using namespace ::com::sun::star::uno;

class BasicBrowserLang: public BrowserLang {
    SvLBoxEntry* insertMethod(SvTreeListBox& t, SvLBoxEntry* parent, const Reference<XIdlMethod>& m) const;
    
    SvLBoxEntry* insertProperty(SvTreeListBox& t, SvLBoxEntry* parent, const Property& p) const;

    SvLBoxEntry* insertAttribute(SvTreeListBox& t, SvLBoxEntry* parent, const Reference<XIdlField> & p) const;
};


static const BasicBrowserLang instance;
const BrowserLang* basicLang = &instance;


static void basicPrintType(OUStringBuffer& str, const Reference<XIdlClass>& type)
{
    // str.appendAscii(RTL_CONSTASCII_STRINGPARAM("C:"));
    // str.append((sal_Int32) type->getTypeClass());
    // str.appendAscii(RTL_CONSTASCII_STRINGPARAM(":"));
    switch (type->getTypeClass()) {
	case TypeClass_SHORT:
	case TypeClass_UNSIGNED_SHORT:
	    str.appendAscii(RTL_CONSTASCII_STRINGPARAM("Integer"));
	    break;
	case TypeClass_LONG:
	case TypeClass_UNSIGNED_LONG:
	    str.appendAscii(RTL_CONSTASCII_STRINGPARAM("Long"));
	    break;
	case TypeClass_FLOAT:
	    str.appendAscii(RTL_CONSTASCII_STRINGPARAM("Single"));
	    break;
	case TypeClass_STRING:
	    str.appendAscii(RTL_CONSTASCII_STRINGPARAM("String"));
	    break;
	case TypeClass_SEQUENCE:
	    basicPrintType(str, type->getComponentType());
	    str.appendAscii(RTL_CONSTASCII_STRINGPARAM("()"));
	    break;
	default:
	    str.append(type->getName());
    }
}


/* Stupid code duplication XIdlClass vs Type*/
static void basicPrintType(OUStringBuffer& str, const typelib_TypeDescriptionReference* type)
{
    switch (type->eTypeClass) {
	case TypeClass_SHORT:
	case TypeClass_UNSIGNED_SHORT:
	    str.appendAscii(RTL_CONSTASCII_STRINGPARAM("Integer"));
	    break;
	case TypeClass_LONG:
	case TypeClass_UNSIGNED_LONG:
	    str.appendAscii(RTL_CONSTASCII_STRINGPARAM("Long"));
	    break;
	case TypeClass_FLOAT:
	    str.appendAscii(RTL_CONSTASCII_STRINGPARAM("Single"));
	    break;
	case TypeClass_STRING:
	    str.appendAscii(RTL_CONSTASCII_STRINGPARAM("String"));
	    break;
	case TypeClass_SEQUENCE:
	    basicPrintType(str, ((typelib_IndirectTypeDescription*)type->pType)->pType);
	    str.appendAscii(RTL_CONSTASCII_STRINGPARAM("()"));
	    break;
	default:
	    str.append(OUString(type->pTypeName));
    }
    
}

static void basicPrintType(OUStringBuffer& str, const Type& type)
{
    basicPrintType(str, type.getTypeLibType());
}

static OUStringBuffer methodString(const Reference<XIdlMethod>& m)
{
    OUStringBuffer str;
    bool is_sub;
    is_sub = m->getReturnType()->getTypeClass() == TypeClass_VOID;
    if (is_sub) {
	str.appendAscii(RTL_CONSTASCII_STRINGPARAM("Sub "));
    } else {
	str.appendAscii(RTL_CONSTASCII_STRINGPARAM("Function "));
    }
    str.append(m->getName());
    str.appendAscii(RTL_CONSTASCII_STRINGPARAM("("));
    {
	Sequence<ParamInfo> param_infos = m->getParameterInfos();
	for (unsigned int i = 0; i < (unsigned int) param_infos.getLength(); i++) {
	    if (i > 0) {
		str.appendAscii(RTL_CONSTASCII_STRINGPARAM(", "));
	    }
	    switch(param_infos[i].aMode) {
		case (ParamMode_IN):
		    str.appendAscii("ByVal ");
		    break;
		case (ParamMode_OUT):
		case (ParamMode_INOUT):
		    str.appendAscii("ByRef ");
		    break;
	    }
	    str.appendAscii(RTL_CONSTASCII_STRINGPARAM(" "));
	    str.append(param_infos[i].aName);
	    str.appendAscii(RTL_CONSTASCII_STRINGPARAM(" As "));
	    basicPrintType(str, param_infos[i].aType);
	}
    }
    str.appendAscii(RTL_CONSTASCII_STRINGPARAM(")"));
    if (!is_sub) {
	str.appendAscii(RTL_CONSTASCII_STRINGPARAM(" As "));
	basicPrintType(str, m->getReturnType());
    }
    return str;
}

SvLBoxEntry* BasicBrowserLang::insertMethod(SvTreeListBox& t, SvLBoxEntry* parent, const Reference<XIdlMethod>& m) const
{
    OUString s = methodString(m).makeStringAndClear();
    return t.InsertEntry(XubString(s), parent);
}

SvLBoxEntry*  BasicBrowserLang::insertProperty(SvTreeListBox& t, SvLBoxEntry* parent, const Property& p) const
{
    OUStringBuffer str;
    str.appendAscii(RTL_CONSTASCII_STRINGPARAM("Dim "));
    str.append(p.Name);
    str.appendAscii(RTL_CONSTASCII_STRINGPARAM(" As "));
    if (p.Attributes != 0) {
	using namespace ::com::sun::star::beans::PropertyAttribute;
	const struct property_attribute {
	    short bit;
	    OUString name;
	} attributes[] = {
	    {MAYBEVOID, OUString(RTL_CONSTASCII_USTRINGPARAM("MAYBEVOID"))},
	    {BOUND, OUString(RTL_CONSTASCII_USTRINGPARAM("BOUND"))},
	    {CONSTRAINED, OUString(RTL_CONSTASCII_USTRINGPARAM("CONSTRAINED"))},
	    {TRANSIENT, OUString(RTL_CONSTASCII_USTRINGPARAM("TRANSIENT"))},
	    {READONLY, OUString(RTL_CONSTASCII_USTRINGPARAM("READONLY"))},
	    {MAYBEAMBIGUOUS, OUString(RTL_CONSTASCII_USTRINGPARAM("MAYBEAMBIGUOUS"))},
	    {MAYBEDEFAULT, OUString(RTL_CONSTASCII_USTRINGPARAM("MAYBEDEFAULT"))},
	    {REMOVEABLE, OUString(RTL_CONSTASCII_USTRINGPARAM("REMOVEABLE"))},
	    {OPTIONAL, OUString(RTL_CONSTASCII_USTRINGPARAM("OPTIONAL"))}};
	
	bool first = TRUE;
	str.appendAscii(RTL_CONSTASCII_STRINGPARAM("["));
	for (unsigned int i = 0; i < sizeof(attributes)/sizeof(property_attribute); i++) {
	    if ((p.Attributes & attributes[i].bit) != 0) {
		if (!first) {
		    str.appendAscii(RTL_CONSTASCII_STRINGPARAM(" ,"));
		}
		str.append(attributes[i].name);
		first = false;
	    }
	}
	str.appendAscii(RTL_CONSTASCII_STRINGPARAM("] "));
    }
    basicPrintType(str, p.Type);
    return t.InsertEntry(XubString(str.makeStringAndClear()), parent);
}

SvLBoxEntry* BasicBrowserLang::insertAttribute(SvTreeListBox& t, SvLBoxEntry* parent, const Reference<XIdlField> & p) const
{
    OUStringBuffer str;
    FieldAccessMode m = p->getAccessMode();
    str.appendAscii(RTL_CONSTASCII_STRINGPARAM("Dim "));
    basicPrintType(str, p->getType());
    str.appendAscii(RTL_CONSTASCII_STRINGPARAM(" As "));
    if (m == FieldAccessMode_READONLY) {
	str.appendAscii(RTL_CONSTASCII_STRINGPARAM("readonly "));
    } else if (m == FieldAccessMode_WRITEONLY) {
	str.appendAscii(RTL_CONSTASCII_STRINGPARAM("writeonly "));
    }
    str.append(p->getName());
    return t.InsertEntry(XubString(str.makeStringAndClear()), parent);
}


};
