/* Copyright (c) Sun Microsystems 2006. All rights reserved.
   This file is part of OpenOffice.org project.
   
   Written by Ramón García Fernández <ramon.garcia.f+openoffice@gmail.com>.
   This work was supported by a Google Summer of Code grant.
*/

/* Declaration of the component object */

#ifndef OBJECTBROWSER_IMPL_HXX
#define OBJECTBROWSER_IMPL_HXX

#include <com/sun/star/container/XNameContainer.hpp>
#include <com/sun/star/lang/XServiceInfo.hpp>
#include <com/sun/star/uno/XComponentContext.hpp>

#include <com/sun/star/reflection/XIdlReflection.hpp>
#include <com/sun/star/beans/XIntrospection.hpp>

#include <com/sun/star/reflection/XObjectBrowser.hpp>

#include <rtl/ustring.hxx>
#include <cppuhelper/implbase3.hxx>

#include <vcl/wrkwin.hxx>
#include <svtools/svtreebx.hxx>
#include <vcl/button.hxx>

#include <osl/mutex.hxx>

#include <hash_map>

/* Object browser data structure,
   including data needed for drawing the tree
*/


namespace objectbrowser_impl {
    using com::sun::star::container::XNameContainer;
    using com::sun::star::uno::Reference;
    using com::sun::star::uno::Sequence;
    using com::sun::star::uno::XComponentContext;
    using com::sun::star::lang::XTypeProvider;
    using com::sun::star::lang::XServiceInfo;
    using com::sun::star::reflection::XIdlReflection;
    using com::sun::star::reflection::XIdlMethod;
    using com::sun::star::reflection::XIdlClass;
    using com::sun::star::beans::XIntrospection;
    using com::sun::star::beans::XIntrospectionAccess;

    using com::sun::star::reflection::XObjectBrowser;
    using com::sun::star::reflection::ObjectBrowserLang;
    
    using com::sun::star::uno::Any;
    using com::sun::star::uno::Type;

    using com::sun::star::uno::RuntimeException;
    using com::sun::star::lang::IllegalArgumentException;
    using com::sun::star::lang::WrappedTargetException;
    using com::sun::star::container::ElementExistException;
    using com::sun::star::container::NoSuchElementException;

    using rtl::OUString;
    using rtl::OUStringHash;

    using osl::Mutex;
    using osl::Guard;

    class ObjectBrowserImpl
	: public ::cppu::WeakImplHelper3<XNameContainer,XServiceInfo,XObjectBrowser>
    {
    public:
	ObjectBrowserImpl(const Reference<XComponentContext> & xContext);

	// XElementAccess
	Type SAL_CALL getElementType() throw (RuntimeException);
	sal_Bool SAL_CALL hasElements() throw (RuntimeException);

	// XNameAccess
	Any SAL_CALL getByName(const OUString& aName) 
	    throw (NoSuchElementException, WrappedTargetException, 
		   RuntimeException);
	Sequence<OUString> SAL_CALL getElementNames() 
	    throw (RuntimeException);
	sal_Bool SAL_CALL hasByName(const OUString& aName) 
	    throw(RuntimeException);

	// XNameContainer
	void SAL_CALL insertByName(const OUString& name, const Any& element) 
	    throw (IllegalArgumentException, ElementExistException, 
		   WrappedTargetException, 
		   RuntimeException);
	void SAL_CALL removeByName(const OUString& name) 
	    throw (NoSuchElementException, WrappedTargetException, 
		   RuntimeException);

	// XNameReplace
	void SAL_CALL replaceByName(const OUString& name, const Any& element)
	    throw (IllegalArgumentException, NoSuchElementException, 
		   WrappedTargetException, RuntimeException);

	// XServiceInfo
	OUString SAL_CALL getImplementationName()
	    throw (RuntimeException);
	sal_Bool SAL_CALL supportsService(OUString const & serviceName )
	    throw (RuntimeException);
	Sequence<OUString> SAL_CALL getSupportedServiceNames()
	    throw (RuntimeException);

	// XObjectBrowser

	ObjectBrowserLang SAL_CALL getlang() throw (RuntimeException);
	void SAL_CALL setlang(ObjectBrowserLang _lang) throw (RuntimeException);

    private:

	osl::Mutex mutex;
	
	typedef std::hash_map<OUString, SvLBoxEntry*, OUStringHash> interface_to_entry_t;

	typedef std::hash_map<OUString, Reference<XIdlClass>, OUStringHash > method_map_t;

	struct object_data {
	    const Any* obj;
	    SvLBoxEntry* tree_item;
	    Sequence<Reference<XIdlClass> > interfaces;
	    Sequence<OUString> services;
	    Reference<XIntrospectionAccess> inspect;
	    method_map_t method_map;
	    interface_to_entry_t interface_to_entry;
	};

	std::hash_map<OUString, object_data, OUStringHash> objects;


	ObjectBrowserLang current_lang;


	class MyTreeListBox: public SvTreeListBox {
	public:
	    MyTreeListBox(WorkWindow* parent, WinBits nStyle, ObjectBrowserImpl* outer) :
		SvTreeListBox(parent, nStyle), outer(outer){
		EnableContextMenuHandling(TRUE);
	    }
	    PopupMenu* CreateContextMenu();
	    void ExcecuteContextMenuAction(USHORT selected_entry);
	private:
	    ObjectBrowserImpl* outer;
	};

	class MyWin: public WorkWindow {
	private:
	    ObjectBrowserImpl* outer;
	    void Resize();
	public:
	    MyTreeListBox tree;
	    RadioButton lang_uno, lang_basic;
	    MyWin(ObjectBrowserImpl* outer);
	    DECL_DLLPRIVATE_LINK(SelectLang, void*);
	    DECL_DLLPRIVATE_LINK(OpenOnlineDoc, void*);
	};


	const Reference<XComponentContext> m_xContext;
	
	const Reference<XIdlReflection> reflection;
	const Reference<XIntrospection> introspection;

	MyWin w;

	SvTreeListBox& tree;

	
	friend void createMethodMap(struct ObjectBrowserImpl::object_data& od);

	friend void insertInterface(SvTreeListBox& t, SvLBoxEntry* parent, const Reference<XIdlClass>& c, struct object_data& od, ObjectBrowserLang current_lang);

	void fillTree(struct object_data& od);

	friend class BrowserLang;
	friend class BasicBrowserLang;
	friend class UnoBrowserLang;

    };
}

#endif
